
_MAJOR_VERSION = 0
_MINOR_VERSION = 4
_MICRO_VERSION = 1
# _RELEASE_DATE = '.'
# _RELEASE_TYPE = 'alpha'
__version__ = '.'.join(
    map(str,
        [_MAJOR_VERSION, _MINOR_VERSION, _MICRO_VERSION])
)

__package_name__ = 'mskit'
__author__ = 'Lou, R.'
__email__ = 'lourh@shanghaitech.edu.cn'
__repository__ = 'https://github.com/gureann/MSKit'
__license__ = 'MIT'

__short_description__ = 'A tool-kit for MS data processing'
