
PRM_Header = [
    'Mass [m/z]',  # MS1 m/z
    'Formula [M]',
    'Formula type',
    'Species',
    'CS [z]',  # Integer
    'Polarity',  # "Positive"
    'Start [min]',
    'End [min]',
    '(N)CE',
    '(N)CE type',
    'MSX ID',
    'Comment',
]
