class BasicpDeepInfo(object):
    pDeepTrainsetTitle = [
        'spec',
        'peptide',
        'modinfo',
        'b',
        'b-NH3',
        'b-H2O',
        'b-ModLoss',
        'y',
        'y-NH3',
        'y-H2O',
        'y-ModLoss']


MOD = {
    'Carbamidomethyl (C)': 'Carbamidomethyl[C]',
    'Oxidation (M)': 'Oxidation[M]',
}
