from mskit.plot_kit.colors import PreColorDict
from mskit.plot_kit.drawing_area import ax_utls
from mskit.plot_kit.annotations import bar_anno, dyna_bar_anno

from matplotlib import pyplot as plt


# Basic inten plot -> combine to mirror plot

def mirror_plot(upper_data, lower_data, colors=None, ls=None, lw=1, ):
    pass
